import argparse
import json
from deepclas4bio import Evaluator
from deepclas4bio.PredictorFactory import PredictorFactory
import importlib

def evaluate(readDataset,path,pathLabels,measures,predictors):
    # with open(config) as file:
    #     data=json.load(file)
    #
    #
    # readDataset=data['readDataset']
    # path = data['pathDataset']
    # pathLabels = data['pathLabels']

    classReadDataset__ = getattr(importlib.import_module('deepclas4bio.'+readDataset), readDataset)
    rd=classReadDataset__()

    evaluator = Evaluator.Evaluator(rd, path,
                                    pathLabels)

    # Anadimos las medidas
    # measures=data['measures']
    for measure in measures:
        evaluator.addMeasure(measure)

    # Anadimos los constructores
    m = PredictorFactory()
    # predictors=data['predictors']
    for predictor in predictors:

        model = m.getPredictor(predictor['framework'], predictor['model'])
        evaluator.addPredictor(model)

    return evaluator.evaluate()



# if __name__=="__main__":
#     parser = argparse.ArgumentParser()
#     parser.add_argument("-c", "--config", required=True, help="Configuration file")
#     args=vars(parser.parse_args())
#     evaluate(args["config"])



from deepclas4bio import DatasetManager, Measures
import importlib
import json


class Evaluator:
    def __init__(self,readDataset,path,pathLabels,batch=64):
        self.readDataset=readDataset
        [self.images,self.labels]=readDataset.readDataset(path,pathLabels)
        self.predictors=[]
        self.measures=[]
        self.batch=batch
        if len(list(set(self.labels))) <2:
            raise ValueError('Only one class detected. Review your dataset.')
        elif len(list(set(self.labels))) == 2:
            self.binary=True
        else:
            self.binary=False


    def addPredictor(self,predictor):
        self.predictors.append(predictor)

    def addMeasure(self, measure):
        self.measures.append(measure)

    def evaluate(self):
        result={}
        for predictor in self.predictors:
            if hasattr(predictor.model,'name'):
                predictorName=predictor.model.name
            else:
                predictorName='DL4J'+predictor.model
            dataManager= DatasetManager.DatasetManager(self.images, batch=self.batch)
            predictions=[]
            while(dataManager.hasNextBach()):
                batchImages=dataManager.nextBatch()
                prediction=predictor.predictBatch(batchImages)
                predictions+=prediction
            resultMeasure={}
            # Mirar si las medidas son adecuadas, binarias o no
            if self.binary:
                allowedMeasures= Measures.binaryMeasures
            else:
                allowedMeasures= Measures.noBinaryMeasures
            for measure in self.measures:
                if measure in allowedMeasures:
                    measureMethod_ = getattr(importlib.import_module('deepclas4bio.'+'Measures'), measure)
                    r = measureMethod_(predictions, self.labels)
                    resultMeasure[measure]=r
            result[predictorName]=resultMeasure
        return result
        # with open('data.json', 'w') as f:
        #     json.dump(result, f)


from deepclas4bio import DatasetManager
from deepclas4bio.PredictorFactory import PredictorFactory

def predict(image,framework,model):
    predictor_factory=PredictorFactory()
    modelo=predictor_factory.getPredictor(framework,model)
    return modelo.predict(image)

def predictBatch(images,framework,model,batch=64):
    predictor_factory=PredictorFactory()
    modelo=predictor_factory.getPredictor(framework,model)
    dataManager= DatasetManager.DatasetManager(images, batch=batch)
    predictions=[]
    while(dataManager.hasNextBach()):
        batchImages=dataManager.nextBatch()
        prediction=modelo.predictBatch(batchImages)
        predictions+=prediction
    results=[]
    for p in predictions:
        results.append(modelo.model.postProcessor(p))
    return results

# parser=argparse.ArgumentParser()
# parser.add_argument("-i", "--image", required=True, help="Image to classify")
# parser.add_argument("-f", "--framework", required=True, help="Framework used to classify the image")
# parser.add_argument("-m", "--model", required=True, help="Model used to classify the image")
#
# args=vars(parser.parse_args())
# result=predict(args["image"],args["framework"], args["model"])
#
# data={}
# data['type']='classification'
# data['image']=args["image"]
# data['framework']=args["framework"]
# data['model']=args["model"]
# data['class']=result
# with open('data.json','w') as f:
#     json.dump(data,f)


import json

readDatasets=[{'name':'ReadDatasetFolders','description':'The images have to be organized in folders. Each folder must have the name of the class which the images belong.'}]

if __name__=="__main__":
    data={}
    data['type']='readDatasets'
    data['readDatasets']=readDatasets
    with open('data.json','w') as f:
        json.dump(data,f)


import torchvision.models as torchmodels
from torchvision.models.resnet import BasicBlock

class ResNet34Kvasir(torchmodels.ResNet):
    def __init__(self):
        super().__init__(BasicBlock, [3, 4, 6, 3],num_classes=8)



from sklearn.metrics import precision_score
from sklearn.metrics import recall_score
from sklearn.metrics import f1_score
from sklearn.metrics import jaccard_similarity_score
from sklearn.metrics import matthews_corrcoef
from sklearn.metrics import roc_auc_score

binaryMeasures=['accuracy','precision','recall','f1','jaccardIndex','matthewsCorrelation','auroc']
noBinaryMeasures=['accuracy','rank5']

def accuracy(predictions,trueLabels):
    i=0
    positive=0
    for prediction in predictions:
        if prediction[0]==trueLabels[i]:
            positive+=1
        i+=1
    return positive/i

def rank5(predictions,trueLabels):
    i = 0
    positive = 0
    for prediction in predictions:
        if trueLabels[i] in prediction[:5]:
            positive+=1
        i+=1
    return positive/i

def precision(predictions,trueLabels):
    predictionsBinary=[]
    for prediction in predictions:
        predictionsBinary.append(prediction[0])
    return precision_score(trueLabels,predictionsBinary)

def recall(predictions,trueLabels):
    predictionsBinary = []
    for prediction in predictions:
        predictionsBinary.append(prediction[0])
    return recall_score(trueLabels,predictionsBinary)

def f1(predictions,trueLabels):
    predictionsBinary = []
    for prediction in predictions:
        predictionsBinary.append(prediction[0])
    return f1_score(trueLabels,predictionsBinary)

def jaccardIndex(predictions,trueLabels):
    predictionsBinary = []
    for prediction in predictions:
        predictionsBinary.append(prediction[0])
    return jaccard_similarity_score(trueLabels,predictionsBinary)

def matthewsCorrelation(predictions,trueLabels):
    predictionsBinary = []
    for prediction in predictions:
        predictionsBinary.append(prediction[0])
    return matthews_corrcoef(trueLabels,predictionsBinary)

def auroc(predictions,trueLabels):
    predictionsBinary = []
    for prediction in predictions:
        predictionsBinary.append(prediction[0])
    return roc_auc_score(trueLabels,predictionsBinary)


from deepclas4bio import Predictor
from subprocess import *
import inspect
import os

class DL4JPredictor(Predictor.Predictor):

    def jarWrapper(self,*args):
        process=Popen(['java', '-jar']+list(args),stdout=PIPE,stderr=PIPE)
        result=[]

        for line in process.stdout:
            result.append(line.decode('utf-8').rstrip('\n').rstrip('\n'))
        return result

    def predict(self,image):
        path=inspect.stack()[0][1]
        pos=path.rfind(os.sep)
        path=path[:pos+1]+'PredictDL4J.jar'

        args=[path,self.model.name[:-4],image]
        result=self.jarWrapper(*args)
        return result[0]

    def predictBatch(self,images):
        path = inspect.stack()[0][1]
        pos = path.rfind(os.sep)
        path = path[:pos + 1] + 'PredictDL4J.jar'

        args = [path, self.model.name[:-4]]+images
        result = self.jarWrapper(*args)
        newResult=[]
        for r in result:
            r=r.replace("[","").replace("]","")
            newResult.append(list(map(int,map(float,r.split(",")))))
        return newResult



from keras.models import model_from_json
from keras.preprocessing import image
from keras.applications import *
from keras.applications.vgg16 import preprocess_input as vgg16preprocess
from keras.applications.vgg19 import preprocess_input as vgg19preprocess
from keras.applications.resnet50 import preprocess_input as resnet50preprocess
from keras.applications.inception_v3 import preprocess_input as inceptionv3preprocess
from keras.applications.mobilenet import preprocess_input as mobileNetpreprocess
from keras.applications.xception import preprocess_input as xceptionpreprocess
from keras.applications.densenet import preprocess_input as densenetpreprocess
from keras.applications.inception_resnet_v2 import preprocess_input as inceptionresnetv2preprocess
from keras.applications.nasnet import preprocess_input as nasnetpreprocess

from keras.applications.vgg16 import decode_predictions as vgg16decode
from keras.applications.vgg19 import decode_predictions as vgg19decode
from keras.applications.resnet50 import decode_predictions as resnet50decode
from keras.applications.inception_v3 import decode_predictions as inceptionv3decode
from keras.applications.mobilenet import decode_predictions as mobileNetdecode
from keras.applications.xception import decode_predictions as xceptiondecode
from keras.applications.densenet import decode_predictions as densenetdecode
from keras.applications.inception_resnet_v2 import decode_predictions as inceptionresnetv2decode
from keras.applications.nasnet import decode_predictions as nasnetdecode

from keras.applications.vgg19 import *
from keras.applications.resnet50 import *
from keras.applications.inception_v3 import *
from keras.applications.densenet import *
from keras.applications.inception_resnet_v2 import *
from keras.applications.nasnet import *
from keras.applications.mobilenet import *

import inspect
import os
import numpy as np

# Add your model here
models=['VGG16','VGG19','ResNet','InceptionV3','MobileNet','Xception','InceptionResNetV2', 'DenseNet', 'NASNet']


######## METHODS FOR LOAD MODELS ########
def vgg16kerasload():
    return VGG16()

def vgg19kerasload():
    return VGG19()

def resnet50kerasload():
    return ResNet50()

def inceptionv3kerasload():
    return InceptionV3()

def mobilenetkerasload():
    return MobileNet()

def xceptionkerasload():
    return Xception()

def densenetkerasload():
    return DenseNet()

def inceptionresnetv2kerasload():
    return InceptionResNetV2()

def nasnetkerasload():
    return NASNet()

# Generic method to laod models from name
def loadModel(modelName):
    path=inspect.stack()[0][1]
    pos=path.rfind(os.sep)
    pathModel=path[:pos+1]+'Classification'+os.sep+'model'+os.sep+modelName+'.json'
    json_file=open(pathModel,'r')
    loaded_model_json=json_file.read()
    json_file.close()
    loaded_model=model_from_json(loaded_model_json)
    pathWeights=path[:pos+1]+'Classification'+os.sep+'weights'+os.sep+modelName+'.h5'
    loaded_model.load_weights(pathWeights)
    return loaded_model

def resnetisickerasload():
    return loadModel('ResNetISIC')


######## METHODS FOR PREPROCESS ########
def commonPreProcess(im):
    img=image.load_img(im,target_size=(224,224))
    x=image.img_to_array(img)
    x=np.expand_dims(x,axis=0)
    return x

def vgg16keraspreprocess(im):
    x=commonPreProcess(im)
    return vgg16preprocess(x)

def vgg19keraspreprocess(im):
    x=commonPreProcess(im)
    return vgg19preprocess(x)

def renet50keraspreprocess(im):
    x=commonPreProcess(im)
    return resnet50preprocess(x)

def inceptionv3keraspreprocess(im):
    x=commonPreProcess(im)
    return inceptionv3preprocess(x)

def mobilenetkeraspreprocess(im):
    x=commonPreProcess(im)
    return mobileNetpreprocess(x)

def xceptionkeraspreprocess(im):
    x=commonPreProcess(im)
    return xceptionpreprocess(x)

def densenetkeraspreprocess(im):
    x=commonPreProcess(im)
    return densenetpreprocess(x)

def inceptionresnetv2keraspreprocess(im):
    x=commonPreProcess(im)
    return inceptionresnetv2preprocess(x)

def nasnetkeraspreprocess(im):
    x=commonPreProcess(im)
    return nasnetpreprocess(x)


######## METHODS FOR POSPROCESS ########
def vgg16keraspostprocess(result):
    prediction=vgg16decode(result,top=1)[0]
    return prediction[0][1]

def vgg19keraspostprocess(result):
    prediction=vgg19decode(result,top=1)[0]
    return prediction[0][1]

def resnet50keraspostrocess(result):
    prediction=resnet50decode(result,top=1)[0]
    return prediction[0][1]

def inceptionv3keraspostprocess(result):
    prediction=inceptionv3decode(result,top=1)[0]
    return prediction[0][1]

def mobilenetkeraspostprocess(result):
    prediction=mobileNetdecode(result,top=1)[0]
    return prediction[0][1]

def xceptionkeraspostprocess(result):
    prediction=xceptiondecode(result,top=1)[0]
    return prediction[0][1]

def densenetkeraspostprocess(result):
    prediction=densenetdecode(result,top=1)[0]
    return prediction[0][1]

def inceptionresnetv2keraspostprocess(result):
    prediction=inceptionresnetv2decode(result,top=1)[0]
    return prediction[0][1]

def nasnetkeraspostprocess(result):
    prediction=nasnetdecode(result,top=1)[0]
    return prediction[0][1]



import inspect
import os
import json


def listFrameworks():
    path=inspect.stack()[0][1]
    pos=path.rfind(os.sep)
    path=path[:pos+1]
    frameworks=[]
    if(pos==-1):
        path='.'
    dirs=os.listdir(path=path)

    for d in dirs:
        if "." not in d and not d.startswith("__") and not d.startswith("temp"):
            frameworks.append(d)
    return frameworks


# if __name__=="__main__":
#     data={}
#     data['type']='frameworks'
#     data['frameworks']=listFrameworks()
#     with open('data.json', 'w') as f:
#         json.dump(data,f)


from deepclas4bio.PredictorFactory import PredictorFactory
from deepclas4bio import Evaluator, ReadDatasetFolders

readdataset= ReadDatasetFolders.ReadDatasetFolders()

evaluator= Evaluator.Evaluator(readdataset, '/home/adines/Escritorio/DatasetPrueba', '/home/adines/Escritorio/labels.txt')

# Añadimos las medidas
evaluator.addMeasure('accuracy')
evaluator.addMeasure('rank5')
evaluator.addMeasure('f1')

m=PredictorFactory()

# Keras
model=m.getPredictor('Keras','VGG16')

model2=m.getPredictor('Keras','VGG19')

# DL4J
model3=m.getPredictor('DL4J','ResNet50')

model4=m.getPredictor('DL4J','GoogleNet')

# Añadimos los modelos
evaluator.addPredictor(model)
evaluator.addPredictor(model2)
evaluator.addPredictor(model3)
evaluator.addPredictor(model4)


evaluator.evaluate()



class DatasetManager:
    def __init__(self,images,batch=64):
        self.images=images
        self.batch=batch
        self.batchNumber=0
        self.numImages=len(self.images)

    def hasNextBach(self):
        return self.batchNumber*self.batch<self.numImages

    def nextBatch(self):
        result= self.images[self.batchNumber:self.batchNumber+64]
        self.batchNumber+=1
        return result


import numpy as np
import inspect
import os
# Add your model here
models=['VGG16','VGG19','ResNet50','GoogLeNet']

def vgg16dl4jpreprocess(im):
    pass

def vgg19dl4jpreprocess(im):
    pass

def resnet50dl4jpreprocess(im):
    pass

def googlenetdl4jpreprocess(im):
    pass



def vgg16dl4jload():
    pass

def vgg19dl4jload():
    pass

def resnet50dl4jload():
    pass

def googlenetdl4jload():
    pass


def commonPostProcess(result):
    path = inspect.stack()[0][1]
    pos = path.rfind(os.sep)
    path = path[:pos + 1]
    labels = np.loadtxt(path + "synset_words.txt", str, delimiter='\n')
    result=np.asarray(result)
    return labels[result.argmax()]

def vgg16dl4jpostprocess(result):
    return commonPostProcess(result)

def vgg19dl4jpostprocess(result):
    return commonPostProcess(result)

def resnet50dl4jpostprocess(result):
    return commonPostProcess(result)

def googlenetdl4jpostprocess(result):
    return commonPostProcess(result)



from deepclas4bio import Measures
import json

def listMeasures():
    bm= Measures.binaryMeasures
    nbm= Measures.noBinaryMeasures
    return list(set().union(bm,nbm))


# if __name__=="__main__":
#     data={}
#     data['type']='measures'
#     data['measures']=listMeasures()
#     with open('data.json','w') as f:
#         json.dump(data,f)


import inspect
import os
import numpy as np
import mxnet as mx
from PIL import Image
from mxnet import gluon,nd
from mxnet.gluon.model_zoo import vision

# Add your model here
models=['VGG16','VGG19','CaffeNet','InceptionV3','NiN', 'ResidualNet152', 'ResNet101', 'SqueezeNet']



######## METHODS FOR LOAD MODELS ########

# Generic method to laod models from name
def loadModel(modelName):
    path=inspect.stack()[0][1]
    pos=path.rfind(os.sep)
    pathModel=path[:pos+1]+'Classification'+os.sep+'model'+os.sep+modelName+'.json'
    pathWeights=path[:pos+1]+'Classification'+os.sep+'weights'+os.sep+modelName+'.params'


    ctx = mx.gpu() if mx.test_utils.list_gpus() else mx.cpu()
    deserialized_net=gluon.nn.SymbolBlock.imports(pathModel, ['data'], pathWeights, ctx=ctx)

    return deserialized_net

def vgg16mxnetload():
    net= vision.vgg16(pretrained=True)
    net.hybridize()
    return net

def vgg19mxnetload():
    return loadModel('VGG19')

def caffenetmxnetload():
    return loadModel('CaffeNet')

def inceptionv3mxnetload():
    return loadModel('InceptionV3')

def ninmxnetload():
    return loadModel('NiN')

def residualnet152mxnetload():
    return loadModel('ResidualNet152')

def resnet101mxnetload():
    return loadModel('ResNet101')

def squeezenetmxnetload():
    return loadModel('SqueezeNet')


######## METHODS FOR PREPROCESS ########
def commonPreProcess(im):

    img=mx.image.image.imread(im)
    img=mx.image.image.imresize(img,224,224)
    img=mx.ndarray.swapaxes(img,0,2)

    img=mx.ndarray.cast(img, dtype='float32')
    return img

def vgg16mxnetpreprocess(im):
    return commonPreProcess(im)

def vgg19mxnetpreprocess(im):
    return commonPreProcess(im)

def caffenetmxnetpreprocess(im):
    return commonPreProcess(im)

def inceptionv3mxnetpreprocess(im):
    return commonPreProcess(im)

def ninmxnetpreprocess(im):
    return commonPreProcess(im)

def residualnet152mxnetpreprocess(im):
    return commonPreProcess(im)

def resnet101mxnetpreprocess(im):
    return commonPreProcess(im)

def squeezenetmxnetpreprocess(im):
    return commonPreProcess(im)


######## METHODS FOR POSPROCESS ########
def commonPostProcess(result):
    result=np.squeeze(result)
    result=np.argsort(result)[::-1]
    path=inspect.stack()[0][1]
    pos=path.rfind(os.sep)
    path=path[:pos+1]
    labels=np.loadtxt(path+"synset_words.txt",str,delimiter='\n')
    return labels[result[0]]


def vgg16mxnetpostprocess(result):
    return commonPostProcess(result)

def vgg19mxnetpostprocess(result):
    return commonPostProcess(result)

def caffenetmxnetpostrocess(result):
    return commonPostProcess(result)

def inceptionv3mxnetpostprocess(result):
    return commonPostProcess(result)

def ninmxnetpostprocess(result):
    return commonPostProcess(result)

def residualnet152mxnetpostprocess(result):
    return commonPostProcess(result)

def resnet101mxnetpostprocess(result):
    return commonPostProcess(result)

def squeezenetmxnetpostprocess(result):
    return commonPostProcess(result)



import importlib

class ModelFactory:
    loadedModels={}
    def getModel(self,framework,model):
        modelName=framework+model
        m=self.loadedModels.get(modelName)
        if m is not None:
            return m
        else:
            if framework=='DL4Jg':
                return model
            else:
                class_name=framework+'Model'
                class__=getattr(importlib.import_module('deepclas4bio.'+framework+'.'+class_name),class_name)
                loadModel=model+framework+'load'
                loadModel=loadModel.lower()
                loadModelMethod = getattr(importlib.import_module('deepclas4bio.'+framework + '.' + framework + 'Functions'), loadModel)
                preprocess=model+framework+'preprocess'
                preprocess=preprocess.lower()
                preprocessMethod=getattr(importlib.import_module('deepclas4bio.'+framework+'.'+framework+'Functions'),preprocess)
                postprocess = model + framework + 'postprocess'
                postprocess = postprocess.lower()
                postprocessMethod = getattr(importlib.import_module('deepclas4bio.'+framework + '.' + framework + 'Functions'), postprocess)
                m= class__(model+framework,loadModelMethod,preprocessMethod,postprocessMethod)
                self.loadedModels[modelName]=m
                return m



from deepclas4bio import IPredictor


class Predictor(IPredictor.IPredictor):

    def __init__(self,model):
        self.model=model

    def predict(self,image):
        pass

    def predictBatch(self,images):
        pass


from deepclas4bio import Measures
import importlib
import inspect
import os

def listFrameworks():
    path=inspect.stack()[0][1]
    pos=path.rfind(os.sep)
    path=path[:pos+1]
    frameworks=[]
    if(pos==-1):
        path='.'
    dirs=os.listdir(path=path)

    for d in dirs:
        if "." not in d and not d.startswith("__") and not d.startswith("temp"):
            frameworks.append(d)
    return frameworks


def listModels(framework):
    class_name = framework + 'Functions'
    models = getattr(importlib.import_module('deepclas4bio.'+framework + '.' + class_name), 'models')
    return models


def listMeasures():
    bm= Measures.binaryMeasures
    nbm= Measures.noBinaryMeasures
    return list(set().union(bm,nbm))


from deepclas4bio import Predictor
import mxnet as mx
import numpy as np

class MxNetPredictor(Predictor.Predictor):
    def predict(self,image):
        preProcessor=self.model.preProcessor
        imageProcessed=preProcessor(image)
        deepModel=self.model.deepModel
        imageProcessed=mx.ndarray.expand_dims(imageProcessed, axis=0)
        y_preds=deepModel(imageProcessed)
        postProcessor=self.model.postProcessor
        return postProcessor(y_preds[0].asnumpy())

    def predictBatch(self,images):
        preProcessor = self.model.preProcessor
        data=[]
        for image in images:
            imageProcessed = preProcessor(image)
            data.append(imageProcessed)
        dataStack=mx.nd.stack(*data,axis=0)
        y_pred = self.model.deepModel(dataStack)
        predictions=[]
        for pred in y_pred:
            x = pred.asnumpy()
            x = np.squeeze(x)
            x=np.argsort(x)[::-1]
            predictions.append(x)
        return predictions



from deepclas4bio import Predictor
import numpy as np

class KerasPredictor(Predictor.Predictor):
    def predict(self,image):
        preProcessor=self.model.preProcessor
        imageProcessed=preProcessor(image)
        y_pred=self.model.deepModel.predict(imageProcessed)
        postProcessor=self.model.postProcessor
        return postProcessor(y_pred)

    def predictBatch(self,images):
        preProcessor = self.model.preProcessor
        data=[]
        for image in images:
            imageProcessed = preProcessor(image)
            data.append(imageProcessed)
        dataStack=np.vstack(data)
        y_pred = self.model.deepModel.predict(dataStack)
        predictions=[]
        for pred in y_pred:
            predictions.append(np.expand_dims(pred,axis=0))
        return predictions



import json
import argparse
import importlib


def listModels(framework):
    class_name = framework + 'Functions'
    models = getattr(importlib.import_module(framework + '.' + class_name), 'models')
    return models


# if __name__=="__main__":
#     parser=argparse.ArgumentParser()
#     parser.add_argument("-f", "--framework",required=True, help="Framework to obtain its models")
#     args=vars(parser.parse_args())
#
#     data={}
#     data['type']='models'
#     data['framework']=args["framework"]
#     data['models']=listModels(args["framework"])
#     with open('data.json','w') as f:
#         json.dump(data,f)


import inspect
import os
import numpy as np
import torch
from PIL import Image
import importlib
from torchvision import transforms
from torch.autograd import Variable
import torchvision.models as torchmodels

# Add your model here
models=['VGG11','VGG13','VGG16','VGG19','AlexNet','DenseNet121','DenseNet161','DenseNet169','DenseNet201','InceptionV3',
        'ResNet18','ResNet34','ResNet50','ResNet101','ResNet152','SqueezeNet10','SqueezeNet11','ResNet34Kvasir']



######## METHODS FOR LOAD MODELS ########

# Generic method to laod models from name
def loadModel(modelName):
    framework="PyTorch"
    class_name=modelName
    package_name='deepclas4bio.'+framework+".Classification.model"
    class__=getattr(importlib.import_module(package_name+"."+class_name),class_name)
    modelo=class__()


    path = inspect.stack()[0][1]
    pos = path.rfind(os.sep)
    path=path[:pos+1]


    pathWeights ='Classification' + os.sep + 'weights' + os.sep + modelName + '.pth'
    pathWeights=path+pathWeights
    state=torch.load(pathWeights,torch.device('cpu'))
    modelo.load_state_dict(state_dict=state, strict=False)
    modelo=modelo.eval()
    return modelo


def vgg11pytorchload():
    return torchmodels.vgg11(pretrained=True)

def vgg13pytorchload():
    return torchmodels.vgg13(pretrained=True)

def vgg16pytorchload():
    return torchmodels.vgg16(pretrained=True)

def vgg19pytorchload():
    return torchmodels.vgg19(pretrained=True)

def alexnetpytorchload():
    return torchmodels.alexnet(pretrained=True)

def densenet121pytorchload():
    return torchmodels.densenet121(pretrained=True)

def densenet161pytorchload():
    return torchmodels.densenet161(pretrained=True)

def densenet169pytorchload():
    return torchmodels.densenet169(pretrained=True)

def densenet201pytorchload():
    return torchmodels.densenet201(pretrained=True)

def inceptionv3pytorchload():
    return torchmodels.inception_v3(pretrained=True)

def resnet18pytorchload():
    return torchmodels.resnet18(pretrained=True)

def resnet34pytorchload():
    return torchmodels.resnet34(pretrained=True)

def resnet50pytorchload():
    return torchmodels.resnet50(pretrained=True)

def resnet101pytorchload():
    return torchmodels.resnet101(pretrained=True)

def resnet152pytorchload():
    return torchmodels.resnet152(pretrained=True)

def squeezenet10pytorchload():
    return torchmodels.squeezenet1_0(pretrained=True)

def squeezenet11pytorchload():
    return torchmodels.squeezenet1_1(pretrained=True)

def resnet34kvasirpytorchload():
    return loadModel('ResNet34Kvasir')


######## METHODS FOR PREPROCESS ########
def commonPreProcess(im):
    img_pil=Image.open(im)
    prep=transforms.Compose([
        transforms.CenterCrop(224),
        transforms.ToTensor()
    ])
    img_prep=prep(img_pil)
    img_var=Variable(img_prep)
    img_var=img_var.unsqueeze(0)
    return img_var


def vgg11pytorchpreprocess(im):
    return commonPreProcess(im)

def vgg13pytorchpreprocess(im):
    return commonPreProcess(im)

def vgg16pytorchpreprocess(im):
    return commonPreProcess(im)


def vgg19pytorchpreprocess(im):
    return commonPreProcess(im)


def alexnetpytorchpreprocess(im):
    return commonPreProcess(im)

def densenet121pytorchpreprocess(im):
    return commonPreProcess(im)

def densenet161pytorchpreprocess(im):
    return commonPreProcess(im)

def densenet169pytorchpreprocess(im):
    return commonPreProcess(im)

def densenet201pytorchpreprocess(im):
    return commonPreProcess(im)

def inceptionv3pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet18pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet34pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet50pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet101pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet152pytorchpreprocess(im):
    return commonPreProcess(im)

def squeezenet10pytorchpreprocess(im):
    return commonPreProcess(im)

def squeezenet11pytorchpreprocess(im):
    return commonPreProcess(im)

def resnet34kvasirpytorchpreprocess(im):
    return commonPreProcess(im)


######## METHODS FOR POSPROCESS ########
def commonPostProcess(result):
    path = inspect.stack()[0][1]
    pos = path.rfind(os.sep)
    path = path[:pos + 1]
    labels = np.loadtxt(path + "synset_words.txt", str, delimiter='\n')
    return labels[result.data.numpy().argmax()]


def vgg11pytorchpostprocess(result):
    return commonPostProcess(result)

def vgg13pytorchpostprocess(result):
    return commonPostProcess(result)

def vgg16pytorchpostprocess(result):
    return commonPostProcess(result)

def vgg19pytorchpostprocess(result):
    return commonPostProcess(result)

def alexnetpytorchpostrocess(result):
    return commonPostProcess(result)

def densenet121pytorchpostrocess(result):
    return commonPostProcess(result)

def densenet161pytorchpostrocess(result):
    return commonPostProcess(result)

def densenet169pytorchpostrocess(result):
    return commonPostProcess(result)

def densenet201pytorchpostrocess(result):
    return commonPostProcess(result)

def inceptionv3pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet18pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet34pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet50pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet101pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet152pytorchpostprocess(result):
    return commonPostProcess(result)

def squeezenet10pytorchpostprocess(result):
    return commonPostProcess(result)

def squeezenet11pytorchpostprocess(result):
    return commonPostProcess(result)

def resnet34kvasirpytorchpostprocess(result):
    labels=["dyed-lifted-polyps","normal-z-line","normal-cecum","normal-pylorus","dyed-resection-margins","ulcerative-colitis","polyps","esophagitis"]
    return labels[result.data.numpy().argmax()]



from deepclas4bio.ModelFactory import ModelFactory
import importlib

class PredictorFactory:
    def getPredictor(self,framework,model):
        modelFact=ModelFactory()
        m=modelFact.getModel(framework,model)
        class_name=framework+'Predictor'
        class__ = getattr(importlib.import_module('deepclas4bio.'+framework + '.' + class_name), class_name)
        return class__(m)


import inspect
import os
import numpy as np
#import sys
#sys.path.append(r'C:\Users\adines\Downloads\caffe\python')
import caffe

# Add your model here
models=['VGG16','VGG19','AlexNet','CaffeNet','GoogleNet']

######## METHODS FOR LOAD MODELS ########

# Generic method to laod models from name
def loadModel(modelName):
    path=inspect.stack()[0][1]
    pos=path.rfind(os.sep)
    path=path[:pos+1]
    pathModel='Classification'+os.sep+'model'+os.sep+modelName+'.prototxt'
    pathModel=path+pathModel
    pathWeights='Classification'+os.sep+'weights'+os.sep+modelName+'.caffemodel'
    pathWeights=path+pathWeights
    net=caffe.Classifier(pathModel,pathWeights,
                         mean=np.load(path+'ilsvrc_2012_mean.npy').mean(1).mean(1),
                         channel_swap=(2,1,0),
                         raw_scale=225,
                         image_dims=(255,255))
    return net


def vgg16caffeload():
    return loadModel('VGG16')

def vgg19caffeload():
    return loadModel('VGG19')

def alexnetcaffeload():
    return loadModel('AlexNet')

def caffenetcaffeload():
    return loadModel('CaffeNet')

def googlenetcaffeload():
    return loadModel('GoogleNet')


######## METHODS FOR PREPROCESS ########
def commonPreProcess(im):
    input_image=caffe.io.load_image(im)
    return input_image

def vgg16caffepreprocess(im):
    return commonPreProcess(im)

def vgg19caffepreprocess(im):
    return commonPreProcess(im)

def alexnetcaffepreprocess(im):
    return commonPreProcess(im)

def caffenetcaffepreprocess(im):
    return commonPreProcess(im)

def googlenetcaffepreprocess(im):
    return commonPreProcess(im)


######## METHODS FOR POSPROCESS ########
def commonPostProcess(result):
    path = inspect.stack()[0][1]
    pos = path.rfind(os.sep)
    path = path[:pos + 1]
    labels = np.loadtxt(path + "synset_words.txt", str, delimiter='\n')
    return labels[result[0].argmax()]

def vgg16caffepostprocess(result):
    return commonPostProcess(result)

def vgg19caffepostprocess(result):
    return commonPostProcess(result)

def alexnetcaffepostrocess(result):
    return commonPostProcess(result)

def caffenetcaffepostprocess(result):
    return commonPostProcess(result)

def googlenetcaffepostprocess(result):
    return commonPostProcess(result)


from deepclas4bio import IReadDataset
import glob
import os
import numpy as np

class ReadDatasetFolders(IReadDataset.IReadDataset):
    def readDataset(self,path,pathLabels):
        folders=glob.glob(path+os.sep+'*')
        images=[]
        groundTruth=[]
        labels = np.loadtxt(pathLabels, str, delimiter='\n')
        for folder in folders:
            pos=folder.rfind(os.sep)
            imagesFolder=glob.glob(folder+os.sep+'*')
            for image in imagesFolder:
                images.append(image)
                groundTruth.append(list(labels).index(folder[pos+1:]))
        return [images,groundTruth]



import argparse
import json
from deepclas4bio import DatasetManager
from deepclas4bio.PredictorFactory import PredictorFactory

def predictBatch(images,framework,model,batch=64):
    predictor_factory=PredictorFactory()
    modelo=predictor_factory.getPredictor(framework,model)
    dataManager= DatasetManager.DatasetManager(images, batch=batch)
    predictions=[]
    while(dataManager.hasNextBach()):
        batchImages=dataManager.nextBatch()
        prediction=modelo.predictBatch(batchImages)
        predictions+=prediction
    results=[]
    for p in predictions:
        results.append(modelo.model.postProcessor(p))
    return results

# parser=argparse.ArgumentParser()
# parser.add_argument("-c", "--config", required=True, help="Path to the config.json file")
#
# args=vars(parser.parse_args())
# with open(args["config"]) as file:
#         data=json.load(file)
#
# framework=data['framework']
# model=data['model']
# images=data['images']
#
# results=predictBatch(images,framework,model)
#
# resultImages=[]
# for image, result in zip(images,results):
#     object={}
#     object['image']=image
#     object['class']=result
#     resultImages.append(object)
#
# data={}
# data['type']='classification'
# data['framework']=framework
# data['model']=model
# data['results']=resultImages
# with open('data.json','w') as f:
#     json.dump(data,f)



from deepclas4bio import Predictor


class PyTorchPredictor(Predictor.Predictor):
    def predict(self,image):
        preProcessor=self.model.preProcessor
        imageProcessed=preProcessor(image)
        deepModel=self.model.deepModel
        y_preds=deepModel(imageProcessed)
        postProcessor=self.model.postProcessor
        return postProcessor(y_preds)

    def predictBatch(self,images):
        preProcessor = self.model.preProcessor
        deepModel = self.model.deepModel
        predictions = []
        for image in images:
            imageProcessed = preProcessor(image)
            y_preds = deepModel(imageProcessed)
            predictions.append(y_preds)
        return predictions
